@@ -18,11 +18,11 @@ from botocore.exceptions import ClientError
 from common.schedulers.slurm_commands import update_nodes
 from common.utils import grouper
 from slurm_plugin.common import log_exception, print_with_count
+from slurm_plugin.fleet_manager import EC2Instance, FleetManagerFactory
 from slurm_plugin.slurm_resources import (
     EC2_HEALTH_STATUS_UNHEALTHY_STATES,
     EC2_INSTANCE_ALIVE_STATES,
     EC2_SCHEDULED_EVENT_CODES,
-    EC2Instance,
     EC2InstanceHealthState,
     InvalidNodenameError,
     parse_nodename,
@@ -54,13 +54,15 @@ class InstanceManager:
         use_private_hostname=False,
         head_node_private_ip=None,
         head_node_hostname=None,
-        instance_name_type_mapping=None,
+        fleet_config=None,
+        run_instances_overrides=None,
+        create_fleet_overrides=None,
     ):
         """Initialize InstanceLauncher with required attributes."""
         self._region = region
         self._cluster_name = cluster_name
         self._boto3_config = boto3_config
-        self.failed_nodes = []
+        self.failed_nodes = {}
         self._ddb_resource = boto3.resource("dynamodb", region_name=region, config=boto3_config)
         self._table = self._ddb_resource.Table(table_name) if table_name else None
         self._hosted_zone = hosted_zone
@@ -68,11 +70,13 @@ class InstanceManager:
         self._use_private_hostname = use_private_hostname
         self._head_node_private_ip = head_node_private_ip
         self._head_node_hostname = head_node_hostname
-        self._instance_name_type_mapping = instance_name_type_mapping or {}
+        self._fleet_config = fleet_config
+        self._run_instances_overrides = run_instances_overrides or {}
+        self._create_fleet_overrides = create_fleet_overrides or {}
 
     def _clear_failed_nodes(self):
         """Clear and reset failed nodes list."""
-        self.failed_nodes = []
+        self.failed_nodes = {}
 
     def add_instances_for_nodes(
         self, node_list, launch_batch_size, update_node_address=True, all_or_nothing_batch=False
@@ -81,28 +85,48 @@ class InstanceManager:
         # Reset failed_nodes
         self._clear_failed_nodes()
         instances_to_launch = self._parse_requested_instances(node_list)
-        for queue, queue_instances in instances_to_launch.items():
-            for instance_type, slurm_node_list in queue_instances.items():
+        for queue, compute_resources in instances_to_launch.items():
+            for compute_resource, slurm_node_list in compute_resources.items():
                 logger.info("Launching instances for slurm nodes %s", print_with_count(slurm_node_list))
+
+                # each compute resource can be configured to use create_fleet or run_instances
+                fleet_manager = FleetManagerFactory.get_manager(
+                    self._cluster_name,
+                    self._region,
+                    self._boto3_config,
+                    self._fleet_config,
+                    queue,
+                    compute_resource,
+                    all_or_nothing_batch,
+                    self._run_instances_overrides,
+                    self._create_fleet_overrides,
+                )
                 for batch_nodes in grouper(slurm_node_list, launch_batch_size):
                     try:
-                        launched_instances = self._launch_ec2_instances(
-                            queue, instance_type, len(batch_nodes), all_or_nothing_batch=all_or_nothing_batch
-                        )
+                        launched_instances = fleet_manager.launch_ec2_instances(len(batch_nodes))
+
                         if update_node_address:
                             assigned_nodes = self._update_slurm_node_addrs(list(batch_nodes), launched_instances)
                             try:
                                 self._store_assigned_hostnames(assigned_nodes)
                                 self._update_dns_hostnames(assigned_nodes)
                             except Exception:
-                                self.failed_nodes.extend(list(assigned_nodes.keys()))
+                                self._update_failed_nodes(set(assigned_nodes.keys()))
+                    except ClientError as e:
+                        logger.error(
+                            "Encountered exception when launching instances for nodes %s: %s",
+                            print_with_count(batch_nodes),
+                            e,
+                        )
+                        error_code = e.response.get("Error", {}).get("Code")
+                        self._update_failed_nodes(set(batch_nodes), error_code)
                     except Exception as e:
                         logger.error(
                             "Encountered exception when launching instances for nodes %s: %s",
                             print_with_count(batch_nodes),
                             e,
                         )
-                        self.failed_nodes.extend(batch_nodes)
+                        self._update_failed_nodes(set(batch_nodes))
 
     def _update_slurm_node_addrs(self, slurm_nodes, launched_instances):
         """Update node information in slurm with info from launched EC2 instance."""
@@ -131,18 +155,21 @@ class InstanceManager:
                     print_with_count(zip(launched_nodes, launched_instances)),
                 )
             if fail_launch_nodes:
-                logger.info("Failed to launch instances for following nodes: %s", print_with_count(fail_launch_nodes))
-                self.failed_nodes.extend(fail_launch_nodes)
+                logger.warning(
+                    "Failed to launch instances due to limited EC2 capacity for following nodes: %s",
+                    print_with_count(fail_launch_nodes),
+                )
+                self._update_failed_nodes(set(fail_launch_nodes), "LimitedInstanceCapacity")
 
             return dict(zip(launched_nodes, launched_instances))
 
         except subprocess.CalledProcessError:
-            logger.info(
-                "Encountered error when updating node %s with instance %s",
+            logger.error(
+                "Encountered error when updating nodes %s with instances %s",
                 print_with_count(slurm_nodes),
                 print_with_count(launched_instances),
             )
-            self.failed_nodes.extend(slurm_nodes)
+            self._update_failed_nodes(set(slurm_nodes))
 
     @log_exception(logger, "saving assigned hostnames in DynamoDB", raise_on_error=True)
     def _store_assigned_hostnames(self, nodes):
@@ -207,61 +234,23 @@ class InstanceManager:
 
     def _parse_requested_instances(self, node_list):
         """
-        Parse out which launch configurations (queue/instance type) are requested by slurm nodes from NodeName.
+        Parse out which launch configurations (queue/compute resource) are requested by slurm nodes from NodeName.
 
-        Valid NodeName format: {queue_name}-{st/dy}-{instance_type}-{number}
-        Sample NodeName: queue1-st-c5_xlarge-2
+        Valid NodeName format: {queue_name}-{st/dy}-{compute_resource_name}-{number}
+        Sample NodeName: queue1-st-computeres1-2
         """
         instances_to_launch = collections.defaultdict(lambda: collections.defaultdict(list))
         for node in node_list:
             try:
-                queue_name, node_type, instance_name = parse_nodename(node)
-                instance_type = self._instance_name_type_mapping[queue_name][instance_name]
-                instances_to_launch[queue_name][instance_type].append(node)
+                queue_name, node_type, compute_resource_name = parse_nodename(node)
+                instances_to_launch[queue_name][compute_resource_name].append(node)
             except (InvalidNodenameError, KeyError):
                 logger.warning("Discarding NodeName with invalid format: %s", node)
-                self.failed_nodes.append(node)
+                self._update_failed_nodes({node}, "InvalidNodenameError")
         logger.debug("Launch configuration requested by nodes = %s", instances_to_launch)
 
         return instances_to_launch
 
-    def _launch_ec2_instances(self, queue, instance_type, current_batch_size, all_or_nothing_batch=False):
-        """Launch a batch of ec2 instances."""
-        try:
-            ec2_client = boto3.client("ec2", region_name=self._region, config=self._boto3_config)
-            result = ec2_client.run_instances(
-                # Configure instances to be terminated (as opposed to stopped) when they are
-                # shutdown via a command run on the instance. This enables compute nodes to
-                # self-terminate without requiring the inclusion of TerminateInstances permissions
-                # in their instance profiles.
-                InstanceInitiatedShutdownBehavior="terminate",
-                # If not all_or_nothing_batch scaling, set MinCount=1
-                # so run_instances call will succeed even if entire count cannot be satisfied
-                # Otherwise set MinCount=current_batch_size so run_instances will fail unless all are launched
-                MinCount=1 if not all_or_nothing_batch else current_batch_size,
-                MaxCount=current_batch_size,
-                # LaunchTemplate is different for every instance type in every queue
-                # LaunchTemplate name format: {cluster_name}-{queue_name}-{instance_type}
-                # Sample LT name: hit-queue1-c5.xlarge
-                LaunchTemplate={
-                    "LaunchTemplateName": f"{self._cluster_name}-{queue}-{instance_type}",
-                    "Version": "$Latest",
-                },
-            )
-
-            return [
-                EC2Instance(
-                    instance_info["InstanceId"],
-                    instance_info["PrivateIpAddress"],
-                    instance_info["PrivateDnsName"].split(".")[0],
-                    instance_info["LaunchTime"],
-                )
-                for instance_info in result["Instances"]
-            ]
-        except ClientError as e:
-            logger.error("Failed RunInstances request: %s", e.response.get("ResponseMetadata").get("RequestId"))
-            raise
-
     def delete_instances(self, instance_ids_to_terminate, terminate_batch_size):
         """Terminate corresponding EC2 instances."""
         ec2_client = boto3.client("ec2", region_name=self._region, config=self._boto3_config)
@@ -269,9 +258,7 @@ class InstanceManager:
         for instances in grouper(instance_ids_to_terminate, terminate_batch_size):
             try:
                 # Boto3 clients retries on connection errors only
-                ec2_client.terminate_instances(
-                    InstanceIds=list(instances),
-                )
+                ec2_client.terminate_instances(InstanceIds=list(instances))
             except ClientError as e:
                 logger.error(
                     "Failed TerminateInstances request: %s", e.response.get("ResponseMetadata").get("RequestId")
@@ -321,7 +308,11 @@ class InstanceManager:
 
     @log_exception(logger, "getting cluster instances from EC2", raise_on_error=True)
     def get_cluster_instances(self, include_head_node=False, alive_states_only=True):
-        """Get instances that are associated with the cluster."""
+        """
+        Get instances that are associated with the cluster.
+
+        Instances without all the info set are ignored and not returned
+        """
         ec2_client = boto3.client("ec2", region_name=self._region, config=self._boto3_config)
         paginator = ec2_client.get_paginator("describe_instances")
         args = {
@@ -333,15 +324,27 @@ class InstanceManager:
             args["Filters"].append({"Name": "tag:parallelcluster:node-type", "Values": ["Compute"]})
         response_iterator = paginator.paginate(PaginationConfig={"PageSize": BOTO3_PAGINATION_PAGE_SIZE}, **args)
         filtered_iterator = response_iterator.search("Reservations[].Instances[]")
-        return [
-            EC2Instance(
-                instance_info["InstanceId"],
-                instance_info["PrivateIpAddress"],
-                instance_info["PrivateDnsName"].split(".")[0],
-                instance_info["LaunchTime"],
-            )
-            for instance_info in filtered_iterator
-        ]
+
+        instances = []
+        for instance_info in filtered_iterator:
+            try:
+                instances.append(
+                    EC2Instance(
+                        instance_info["InstanceId"],
+                        instance_info["PrivateIpAddress"],
+                        instance_info["PrivateDnsName"].split(".")[0],
+                        instance_info["LaunchTime"],
+                    )
+                )
+            except Exception as e:
+                logger.warning(
+                    "Ignoring instance %s because not all EC2 info are available, exception: %s, message: %s",
+                    instance_info["InstanceId"],
+                    type(e).__name__,
+                    e,
+                )
+
+        return instances
 
     def terminate_all_compute_nodes(self, terminate_batch_size):
         try:
@@ -351,3 +354,7 @@ class InstanceManager:
         except Exception as e:
             logger.error("Failed when terminating compute fleet with error %s", e)
             return False
+
+    def _update_failed_nodes(self, nodeset, error_code="Exception"):
+        """Update failed nodes dict with error code as key and nodeset value."""
+        self.failed_nodes[error_code] = self.failed_nodes.get(error_code, set()).union(nodeset)
